//
// GameX - WindowsDX Class Header 
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 
#include <windows.h>
#include <stdio.h>
#include <ddraw.h>

#define GAMEX_HICOLOR555


#include "gamex-debug.hpp"
#include "gamex-image.hpp"

#ifndef WINDX_DEF
	#define WINDX_DEF

	#define WINDX_DEBUG						// Turns on WindowsDX debugging	
	
	#define WINDX_BPP				16
	#ifndef GAMEX_XSIZE
		#define WINDX_XSIZE			640
	#else
		#define WINDX_XSIZE			GAMEX_XSIZE
	#endif
	#ifndef GAMEX_YSIZE
		#define WINDX_YSIZE			480
	#else
		#define WINDX_YSIZE			GAMEX_YSIZE
	#endif 
	#ifdef GAMEX_FULLSCREEN
		#define WINDX_FULLSCREEN	TRUE	// Request full screen (TRUE/FALSE)
	#else
		#define WINDX_FULLSCREEN	FALSE
	#endif

	#ifndef GAMEX_HICOLOR555
		#define WINDX_HIMASKB		31		// 16-bit 5r-5g-5b format 
		#define WINDX_HIMASKG		992
		#define WINDX_HIMASKR		31774
		#define WINDX_HISHIFTG		5
		#define WINDX_HISHIFTR		10
	#else
		#define WINDX_HIMASKB		31		// 16-bit 5r-6g-5b format
		#define WINDX_HIMASKG		2016
		#define WINDX_HIMASKR		63448
		#define WINDX_HISHIFTG		6
		#define WINDX_HISHIFTR		11
	#endif	

	#define WINDX_BUFSIZE			1000	// Keyboard Buffer Size

	#define MOUSE_POS_RATE	1.0
	#define MOUSE_LEFT		2
	#define MOUSE_MIDDLE	3
	#define MOUSE_RIGHT		4

	class WindowsDX {
	public:
		WindowsDX (int full, int xres, int yres);
		~WindowsDX (void);

		//**** Interface Functions (cross-platform)
		// Request Functions
		void UseAlpha (void);		
		// Drawing Functions
		void SetView (int x1, int y1, int x2, int y2);			// Set viewport		
		int DrawOk (void);										// Check if graphics was initialized ok?
		int DrawOpen ();										// Open graphics output
		int DrawClose ();										// Close graphics output
		void DrawClear (int n);									// Clear viewport
		void DrawColor (int r, int g, int b);					// Select drawing color		
		inline int DrawPixel (int x, int y, int r, int g, int b)	{return (*FuncDrawPixel) (x,y,r,g,b);}	// Draw pixel
		inline int DrawImage (int x, int y, ImageX *img)			{return (*FuncDrawImage) (x,y,img);}	// Draw image
		inline int DrawMasked (int x, int y, ImageX *img)			{return (*FuncDrawMasked) (x,y,img);}	// Draw masked image
		inline int DrawAlpha (int x, int y, ImageX *img)			{return (*FuncDrawAlpha) (x,y,img);}	// Draw alpha image
		inline int DrawAdded (int x, int y, ImageX *img)			{return (*FuncDrawAdded) (x,y,img);}	// Draw added image
		int DrawBackground (int x, int y, ImageX *img);
		
		int DrawText (int x, int y, char *msg);
		void DrawBox (int x1, int y1, int x2, int y2, int r, int g, int b);
		int ViewUpdate (void);									// Updates view for the next frame		
		// Keyboard Input Functions
		char GetKey (void);
		bool GetKeyDown (int n)			{return win_keydown[n];}
		int GetWidth ();
		int GetHeight ();
		int GetPitch ();
		void *GetScreen (void);
		HWND GetWindow ();		
		// Mouse Input Functions
		int GetMousePress ();
		int GetMouseX ();
		int GetMouseY ();
		float GetMouseDX ();
		float GetMouseDY ();

		//*** Windows Control Functions
		// Initialization & Clean Up Functions
		int InitWindow (HINSTANCE hThisInst, int nWinMode, char *szWinName);
		int InitDirect (void);
		void DirectDrawError (int status);
		int CleanUp (void);		
		// Internal State Control Functions
		void SetActivation (BOOL act);
		void SetPosition (int x, int y);		
		void SetMouseMove (int x, int y);
		void SetMousePress (int button);
		void SetMouseRelease ();
		void SetKey (char ch);
		void SetKeyDown (char ch)		{win_keydown[ch] = true;}
		void SetKeyUp (char ch)			{win_keydown[ch] = false;}
		
		// Clipping Functions
		void ClipTile (int x, int y, int xres, int yres, int &mx, int &my, int &offx, int &offy);
		bool ClipImage (int x, int y, ImageX *img, int &view_offset, int &img_offset, int &img_xs, int &img_ys);

		// Public Variables (for Static Draw Function Switching)
		char	*win_screen;
		int		win_ytable[1024];
		int		win_height, win_width, win_pitch;
	private:		
		//**** Private Functions		
		// Dispatched Drawing Functions
		int (*FuncDrawPixel) (int x, int y, int r, int g, int b);
		int (*FuncDrawImage) (int x, int y, ImageX *img);
		int (*FuncDrawMasked) (int x, int y, ImageX *img);
		int (*FuncDrawAlpha) (int x, int y, ImageX *img);
		int (*FuncDrawAdded) (int x, int y, ImageX *img);		
		
		// Windows & DirectX Specific Initialization
		int WindowInitClass (HINSTANCE hThisInst, char *szWinName);				// Create a window class
		int WindowInit (HINSTANCE hThisInst, char *szWinName);	// Create a main window
		int WindowDraw (int nWinMode);							// Draw window on screen
		int DirectDrawInit ();									// Initialize direct draw
		int DirectDrawSurfInit ();								// Initialize direct draw surfaces
		int DirectDrawCleanUp (void);							// Cleanup direct draw

		int						windx_full;
		int						windx_xsize, windx_ysize;
		int						win_fullscreen;	
		int						win_screenx, win_screeny;
		int						win_screenbpp;
		int						win_viewx1, win_viewy1;
		int						win_viewx2, win_viewy2;		
		int						win_size;
		int						win_widbits;
		int						win_xpos, win_ypos;		
		RECT					win_destrect;
		BOOL					win_active;

		HWND					win_hwnd;

		int						win_r, win_g, win_b;
		long int				win_rgb;
		int						win_drawok;
		int						win_keys;
		char					win_keybuf[WINDX_BUFSIZE];
		bool					win_keydown[256];

		int						mouse_x, mouse_y;
		int						mouse_xl, mouse_yl;
		int						mouse_down, mouse_button;
		int						mouse_motion;

		unsigned short *		table_alpha;

		// Direct Draw 8.1
		//LPDIRECT3D8			dd_main;		// D3DDevice
        //LPDIRECT3DDEVICE8		dd_device;		// Rendering device
        //D3DCAPS8 *			dd_caps;		// Caps for the Rendering Device
		//LPD3DXSPRITE			dd_sprite;		// Sprites
		//LPDIRECT3DTEXTURE8	dd_texture;
		//RECT *				dd_rect;
		
		// Direct Draw 3.0		
		LPDIRECTDRAW			dd_main;
		LPDIRECTDRAWSURFACE		dd_front_buf;
		LPDIRECTDRAWSURFACE		dd_back_buf;
	};

	//*** Switched Drawing Routines
	// 16-bit Drawing Routines
	static int DrawPixel16 (int x, int y, int r, int g, int b);	// Draw pixel (16-bit)
	static int DrawImage16 (int x, int y, ImageX *img);			// Draw image (16-bit)
	static int DrawMasked16 (int x, int y, ImageX *img);		// Draw masked image (16-bit)
	static int DrawAlpha16 (int x, int y, ImageX *img);		// Draw alpha image (16-bit)
	static int DrawAdded16 (int x, int y, ImageX *img);		// Draw added image (16-bit)

	#ifdef GAMEX_MAIN
		WindowsDX GameX (WINDX_FULLSCREEN, WINDX_XSIZE, WINDX_YSIZE);		
	#else
		extern WindowsDX GameX;
		extern void initialize (void);
		extern void main (void);
	#endif


#endif

